#define  	iTSVPath "iTSV.ex5"
#resource	"\\" + iTSVPath

struct SiTSV {
	double							tsv;												// TSV VALUE
	double							ema;												// EMA OF TSV VALUES
	double							slope;											// SLOPE OF EMA
};

//+-----------------------------------------------------------------------------------------------------------------------------+
//| CLASS:        CiTSV
//| APPLICATION:  RELATIVE (PERIODIC) VOLUME INDICATOR
//+-----------------------------------------------------------------------------------------------------------------------------+
class CiTSV {

public:

	// DEFAULT EMPTY CONSTRUCTOR TO SET CLASS WITH CURRENT SYMBOL AND CHART. NOTE: Init() METHOD IS NOT CALLED
    //CiTSV() : CBaseIndicator(_Symbol,PERIOD_CURRENT) {  }

	// NOW USE 'new' OPERATOR FOR PARAMETRIC CONSTRUCTOR TO SET CORRECT PARAMETERS
    CiTSV(string pSymbol,ENUM_TIMEFRAMES pTimeFrame,uint pTSV_SumPeriod,uint pTSV_MAPeriod,ENUM_APPLIED_PRICE pTSV_AppliedPrice);
   ~CiTSV() { IndicatorRelease(mHandleTSV); }

private:

		int									mHandleTSV;
		string							mSymbol;
		ENUM_TIMEFRAMES			mTimeFrame;
		uint								mTSV_SumPeriod;					// TSV SUMMATION PERIOD [13]
    uint								mTSV_MAPeriod;					// TSV AVERAGING PERIOD [7]
    ENUM_APPLIED_PRICE	mTSV_AppliedPrice;			// TSV APPLIED PRICE [PRICE_CLOSE]

public:
    void       					GetArrayTSV(int pCount,SiTSV &pTSV[]);
    double   						CopyBufferTSV(int pBufferNo,int pIndex);

}; // END Of Class declaration
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       CiTSV()
//| APPLICATION:  Parametric Constructor method
//+-----------------------------------------------------------------------------------------------------------------------------+
CiTSV::CiTSV(string pSymbol,ENUM_TIMEFRAMES pTimeFrame,uint pTSV_SumPeriod,uint pTSV_MAPeriod,ENUM_APPLIED_PRICE pTSV_AppliedPrice) {

  	// ASSIGN VALUES TO INDICATOR MEMBER VARIABLES
		mSymbol 					= pSymbol;
		mTimeFrame				= pTimeFrame;
    mTSV_SumPeriod 		= pTSV_SumPeriod;
		mTSV_MAPeriod	 		= pTSV_MAPeriod;
		mTSV_AppliedPrice	= pTSV_AppliedPrice;

		//TesterHideIndicators(true);
    mHandleTSV = iCustom(mSymbol,mTimeFrame,"::" + iTSVPath,mTSV_SumPeriod,mTSV_MAPeriod,PRICE_CLOSE);
    if(mHandleTSV == INVALID_HANDLE) {
			ResetLastError();
 			string vMethod = "[" + mSymbol + "," + EnumToString(mTimeFrame) + "] " + (string)__FUNCTION__;
      PrintFormat("%s Error#[%i] creating handle iTSV",vMethod,GetLastError());
		}
		//TesterHideIndicators(false);

} // END Of parametric constructor()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:  			GetArrayTSV()
//| APPLICATION:  Method to return range of values into parameter passed in as &pVolume
//+-----------------------------------------------------------------------------------------------------------------------------+
void CiTSV::GetArrayTSV(int pCount,SiTSV &pTSV[]) {

		string vMethod = "[" + mSymbol + "," + EnumToString(mTimeFrame) + "] " + (string)__FUNCTION__;
 		string vIndexTime = TimeToString(iTime(mSymbol,mTimeFrame,0),TIME_DATE|TIME_MINUTES);

    if(mHandleTSV == INVALID_HANDLE) {
			CiTSV(mSymbol,mTimeFrame,mTSV_SumPeriod,mTSV_MAPeriod,mTSV_AppliedPrice);
		}

    ArrayResize(pTSV,pCount+1);															// Index[0] added
    ArraySetAsSeries(pTSV,true);

    for(int i = 0; i <= pCount; i++) {											// Copy from Index[0] to keep compatibility with Chart
      pTSV[i].tsv 		= CopyBufferTSV(0,i);
      pTSV[i].ema			= CopyBufferTSV(2,i);
			double prevEMA 	= CopyBufferTSV(2,i+2);
      pTSV[i].slope 	= (pTSV[i].ema - prevEMA) / prevEMA;
    	if(i!=0) PrintFormat("%s [%s] index[%i] TSV[%+i] EMA[%+i] - prevEMA[%+i] = [%+i] / prevEMA[%+i] = Slope[%+.3f]",vMethod,vIndexTime,i,pTSV[i].tsv,pTSV[i].ema,prevEMA,(pTSV[i].ema - prevEMA),prevEMA,pTSV[i].slope);
    }

    return;

} // END Of method GetArrayTSV()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:  			CopyBufferTSV()
//| APPLICATION:  Method to return specified Index value for TSV Indicator
//+-----------------------------------------------------------------------------------------------------------------------------+
double CiTSV::CopyBufferTSV(int pBufferNo,int pIndex) {

    double bufferData[1];       				// static buffer
    if(CopyBuffer(mHandleTSV,pBufferNo,pIndex,1,bufferData) == -1) {
			ResetLastError();
			string vMethod = "[" + (string)mSymbol + "," + EnumToString(mTimeFrame) +"]" + (string)__FUNCTION__;
      PrintFormat("%s Error#[%i] Getting value from iTSV at Buffer[%i] Index[%i]",vMethod,GetLastError(),pBufferNo,pIndex);
      return(0);
    }

    return(bufferData[0]);

} // END Of method CopyBufferTSV()
//+-----------------------------------------------------------------------------------------------------------------------------+
